import {{ properCase tag }} from './{{ tagWithoutPrefix tag }}.component.js';

export * from './{{ tagWithoutPrefix tag }}.component.js';
export default {{ properCase tag }};

{{ properCase tag }}.define('{{ tag }}');

declare global {
    interface HTMLElementTagNameMap {
      '{{ tag }}': {{ properCase tag }};
    }
}
